@@ -20,20 +20,33 @@
  * 
  */
 /**
-    File Name:          7.5-5-n.js
-    ECMA Section:       7.5 Identifiers
-    Description:        Identifiers are of unlimited length
-                        - can contain letters, a decimal digit, _, or $
-                        - the first character cannot be a decimal digit
-                        - identifiers are case sensitive
+    File Name:          7.4.2-16-n.js
+    ECMA Section:       7.4.2
+
+    Description:
+    The following tokens are ECMAScript keywords and may not be used as
+    identifiers in ECMAScript programs.
+
+    Syntax
+
+    Keyword :: one of
+     break          for         new         var
+     continue       function    return      void
+     delete         if          this        while
+     else           in          typeof      with
+
+    This test verifies that the keyword cannot be used as an identifier.
+    Functioinal tests of the keyword may be found in the section corresponding
+    to the function of the keyword.
 
     Author:             christine@netscape.com
-    Date:               11 september 1997
+    Date:               12 november 1997
+
 */
-    var SECTION = "7.5-5-n";
+    var SECTION = "7.4.1-16-n";
     var VERSION = "ECMA_1";
     startTest();
-    var TITLE   = "Identifiers";
+    var TITLE   = "Keywords";
 
     writeHeaderToLog( SECTION + " "+ TITLE);
 
@@ -43,22 +56,20 @@
 function getTestCases() {
     var array = new Array();
     var item = 0;
-
-    array[item++] = new TestCase( SECTION,    "var 0abc",   "error",    "var 3abc" );
+    array[item++] = new TestCase( SECTION,  "var with = true",     "error",    "var with = true" );
     return ( array );
 }
 
 function test() {
     for ( tc=0; tc < testcases.length; tc++ ) {
         testcases[tc].actual = eval( testcases[tc].actual );
-
         testcases[tc].passed = writeTestCaseResult(
                             testcases[tc].expect,
                             testcases[tc].actual,
-                            testcases[tc].description +":  "+
+                            testcases[tc].description +" = "+
                             testcases[tc].actual );
 
-        testcases[tc].reason += ( testcases[tc].passed ) ? "" : " ignored chars after line terminator of single-line comment";
+        testcases[tc].reason += ( testcases[tc].passed ) ? "" : "wrong value ";
     }
     stopTest();
     return ( testcases );
